<?php
/*
Plugin Name: Dynamic Page Menu
Plugin URI: http://www.inworks.jp/
Description: This Plugin dynamically makes the folding menu. 
Author: skatsube
Version: 1.02
Author URI: http://www.inworks.jp/
*/

define('INW_DPM_CONFIG_ALWAYS_DISP_TOPLEVEL', true);

// global
$inw_page_map = array();
$inw_dpm_depth = 0;
$inw_dpm_moredesc = ' (+)';
$inw_show_brother = false;

function wp_list_dynamic_pagemenu($args = '')
{
	global $inw_page_map, $wp_query, $inw_dpm_depth, $inw_dpm_moredesc, $inw_show_brother;

	$defaults = array(
		'depth' => 0, 'show_date' => '',
		'date_format' => get_option('date_format'),
		'child_of' => 0, 'exclude' => '',
		'title_li' => '', 'echo' => 1,
		'authors' => '', 'sort_column' => 'menu_order, post_title', 
		'moredesc' => ' (+)',
		'show_brother' => '0'
	);

	$r = wp_parse_args( $args, $defaults );
	extract( $r, EXTR_SKIP );
	$r['exclude'] = preg_replace('[^0-9,]', '', $r['exclude']);
	$r['exclude'] = implode(',', apply_filters('wp_list_pages_excludes', explode(',', $r['exclude'])));
	$r['hierarchical'] = 0;
	$r['child_of'] = '';
	$inw_dpm_depth = $r['depth'];
	$inw_dpm_moredesc = $r['moredesc'];
	$inw_show_brother = ($r['show_brother'] == 1);
	unset($r['moredesc']);
	unset($r['show_brother']);
	
	$pages = get_pages($r);

	$root =& new CInwPage(null); 
	$inw_page_map['0'] =& $root;
	foreach($pages as $page) {
		$pg =& new CInwPage($page);
	}
	
	// exclude invalid page obj
	$exclude_finish = false;
	while( ! $exclude_finish ) {
		$exclude_finish = true;
		foreach($inw_page_map as $key => $val) {
			if ( $key != '0'  ) {
				$parent =& $inw_page_map[$val->post_parent];
				if ( is_null($parent) || ! is_object($parent)) {
					$inw_page_map = array_diff($inw_page_map, array($key => $val));
					$exclude_finish = false;
				}
			}
		}
	}
	
	// 
	foreach($inw_page_map as $key => $val) {
		if ( $key != '0'  ) {
			$parent =& $inw_page_map[$val->post_parent];
			array_push($parent->children, $key);
			array_unique($parent->children);
		}
	}
	
	// expand menu.
	if ( is_page() || $wp_query->is_posts_page ) {
		$cur_ID = (int)$wp_query->get_queried_object_id();
	
		if ( isset($inw_page_map[$cur_ID])) {
			$cur_page =& $inw_page_map[$cur_ID];
			if (is_object($cur_page)) {
				$cur_page -> OpenPages(true);
				$cur_page -> SetChildrenTag();
				$cur_page -> SetBrotherTag();
			}
		}
	}
	
	if (INW_DPM_CONFIG_ALWAYS_DISP_TOPLEVEL) {
		$root -> OpenChildPages();
	}
	$root -> SetPageLevel(0);
	
	$output = '';
	$root->Render($output);

	if ( $r['echo'] )
		echo $output;
	else
		return $output;	
	
}

class CInwPage
{
	var $parent			= null;
	var $children		= array();
	var $post_title		= '';
	var $ID				= '0';
	var $post_parent	= '0';
	var $visible		= false;
	var $is_child		= false;
	var $is_brother		= false;
	var $page_level		= 0;
	
	function CInwPage($page)
	{
		global $inw_page_map;
		
		if ( ! is_null($page) && is_object($page)) {		
			$this -> ID = $page->ID;
			$this -> post_title = $page->post_title;
			$this -> post_parent = $page->post_parent;
			$inw_page_map[$this->ID] =& $this;
		}
	}
	
	function Render(& $str)
	{
		global $inw_page_map, $wp_query, $inw_dpm_depth, $inw_dpm_moredesc;
		
		if ($this->ID=='0') {
			if ( count($this->children) > 0) {
				foreach( $this -> children as $child_id ) {
					$child =& $inw_page_map[$child_id];
					$child -> Render($str);
				}
			}
			return;			
		}
	
		if ($this ->visible) {

			$css_class = 'page_item page-item-'.$this->ID;
			if ( is_page() || $wp_query->is_posts_page )
				$cur_ID = $wp_query->get_queried_object_id();

			if ( !empty($cur_ID) ) {
				$_current_page = get_page( $cur_ID );
				if ( in_array($this->ID, (array) $_current_page->ancestors) )
					$css_class .= ' current_page_ancestor';
				if ( $this->ID == $cur_ID )
					$css_class .= ' current_page_item';
				elseif ( $_current_page && $this->ID == $_current_page->post_parent )
					$css_class .= ' current_page_parent';

				$css_class .= ' page_level_' . (int)$this -> page_level;
				
				if ( $this->IsInvisibleChildren() ) {
					$css_class .= ' page_has_more_children';
				}
				
				if ($this -> is_child) {
					$css_class .= ' current_page_child';				
				}

				if ($this -> is_brother) {
					$css_class .= ' current_page_brother';				
				}

			}
	
			$str .= '<li class="' . $css_class . '"><a href="' . get_page_link($this -> ID) . '">' . $this -> post_title;
			if ($this->IsInvisibleChildren()) {
				$str .= $inw_dpm_moredesc;
			}
			$str .= '</a></li>' . "\n";
		}
		
		if ( count($this->children) > 0) {
			$str_child = "";
			foreach( $this -> children as $child_id ) {
				$child =& $inw_page_map[$child_id];
				$child -> Render($str_child);
			}
			
			if ($str_child != '') {
				if ( $inw_dpm_depth == '-1') {
					$str .= $str_child;
				}
				else {
					$str .= "<ul>\n" . $str_child . "</ul>\n";
				}
			}
		}
	}
	
	function OpenPages($is_open_child)
	{
		global $inw_page_map;

		$this -> visible = true;
		if ($is_open_child) {
			$this -> OpenChildPages();
		}
		
		if (! empty($this -> post_parent) && isset($inw_page_map[$this -> post_parent]) ) {
			$parent =& $inw_page_map[$this->post_parent];
			$parent -> OpenPages(false);
		}
	}
	
	function OpenChildPages()
	{
		global $inw_page_map;
		foreach($this->children as $child_id)
		{
			if (isset($inw_page_map[$child_id])) {
				$child =& $inw_page_map[$child_id];
				$child -> visible = true;
			}
		}
	}
	
	function IsInvisibleChildren()
	{
		global $inw_page_map;

		if (count($this->children) == 0)
			return false;
		
		foreach($this->children as $child_id)
		{
			if (isset($inw_page_map[$child_id])) {
				$child =& $inw_page_map[$child_id];
				if ( ! $child -> visible ) {
					return true;
				}
			}
		}
		
		return false;
	}
	
	function SetChildrenTag()
	{
		global $inw_page_map;

		if (count($this->children) == 0)
			return;
		
		foreach($this->children as $child_id)
		{
			if (isset($inw_page_map[$child_id])) {
				$child =& $inw_page_map[$child_id];
				$child -> is_child = true;
			}
		}
		
		return;
	}
	
	function SetBrotherTag()
	{
		global $inw_page_map, $inw_show_brother;

		if (! empty($this -> post_parent) && isset($inw_page_map[$this -> post_parent]) ) {
			$parent =& $inw_page_map[$this->post_parent];
			
			if (count($parent->children) == 0)
				return;

			foreach($parent->children as $child_id)
			{
				if (isset($inw_page_map[$child_id]) && $child_id != $this -> ID) {
					$child =& $inw_page_map[$child_id];
					$child -> is_brother = true;
					if ($inw_show_brother) {
						$child -> visible = true;
					}
				}
			}
			
		}
	
		return;
	}
	
	
	function SetPageLevel($my_level)
	{
		global $inw_page_map;

		$this -> page_level = $my_level;

		if (count($this->children) == 0)
			return;
		
		foreach($this->children as $child_id)
		{
			if (isset($inw_page_map[$child_id])) {
				$child =& $inw_page_map[$child_id];
				$child -> SetPageLevel($my_level + 1);
			}
		}
		
		return;		
	}
	
};


?>